/*
 * Copyright (c) 2010 Sascha Hauer <s.hauer@pengutronix.de>
 * Copyright (C) 2005-2009 Freescale Semiconductor, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */
#ifndef __IPU_PRV_H__
#define __IPU_PRV_H__

#include <linux/types.h>
#include <linux/device.h>
#include <linux/clk.h>
#include <linux/platform_device.h>
#include <mach/hardware.h>

#define MX51_IPU_CHANNEL_CSI0			 0
#define MX51_IPU_CHANNEL_CSI1			 1
#define MX51_IPU_CHANNEL_CSI2			 2
#define MX51_IPU_CHANNEL_CSI3			 3
#define MX51_IPU_CHANNEL_MEM_BG_SYNC		23
#define MX51_IPU_CHANNEL_MEM_FG_SYNC		27
#define MX51_IPU_CHANNEL_MEM_DC_SYNC		28
#define MX51_IPU_CHANNEL_MEM_FG_SYNC_ALPHA	31
#define MX51_IPU_CHANNEL_MEM_DC_ASYNC		41
#define MX51_IPU_CHANNEL_ROT_ENC_MEM		45
#define MX51_IPU_CHANNEL_ROT_VF_MEM		46
#define MX51_IPU_CHANNEL_ROT_PP_MEM		47
#define MX51_IPU_CHANNEL_ROT_ENC_MEM_OUT	48
#define MX51_IPU_CHANNEL_ROT_VF_MEM_OUT		49
#define MX51_IPU_CHANNEL_ROT_PP_MEM_OUT		50
#define MX51_IPU_CHANNEL_MEM_BG_SYNC_ALPHA	51

#define IPU_DISP0_BASE		0x00000000
#define IPU_MCU_T_DEFAULT	8
#define IPU_DISP1_BASE		(IPU_MCU_T_DEFAULT << 25)
#define IPU_CM_REG_BASE		0x1e000000
#define IPU_IDMAC_REG_BASE	0x1e008000
#define IPU_ISP_REG_BASE	0x1e010000
#define IPU_DP_REG_BASE		0x1e018000
#define IPU_IC_REG_BASE		0x1e020000
#define IPU_IRT_REG_BASE	0x1e028000
#define IPU_CSI0_REG_BASE	0x1e030000
#define IPU_CSI1_REG_BASE	0x1e038000
#define IPU_DI0_REG_BASE	0x1e040000
#define IPU_DI1_REG_BASE	0x1e048000
#define IPU_SMFC_REG_BASE	0x1e050000
#define IPU_DC_REG_BASE		0x1e058000
#define IPU_DMFC_REG_BASE	0x1e060000
#define IPU_CPMEM_REG_BASE	0x1f000000
#define IPU_LUT_REG_BASE	0x1f020000
#define IPU_SRM_REG_BASE	0x1f040000
#define IPU_TPM_REG_BASE	0x1f060000
#define IPU_DC_TMPL_REG_BASE	0x1f080000
#define IPU_ISP_TBPR_REG_BASE	0x1f0c0000
#define IPU_VDI_REG_BASE	0x1e068000

/* Register addresses */
/* IPU Common registers */
#define IPU_CM_REG(offset)	(offset)

#define IPU_CONF			IPU_CM_REG(0)

#define IPU_SRM_PRI1			IPU_CM_REG(0x00a0)
#define IPU_SRM_PRI2			IPU_CM_REG(0x00a4)
#define IPU_FS_PROC_FLOW1		IPU_CM_REG(0x00a8)
#define IPU_FS_PROC_FLOW2		IPU_CM_REG(0x00ac)
#define IPU_FS_PROC_FLOW3		IPU_CM_REG(0x00b0)
#define IPU_FS_DISP_FLOW1		IPU_CM_REG(0x00b4)
#define IPU_FS_DISP_FLOW2		IPU_CM_REG(0x00b8)
#define IPU_SKIP			IPU_CM_REG(0x00bc)
#define IPU_DISP_ALT_CONF		IPU_CM_REG(0x00c0)
#define IPU_DISP_GEN			IPU_CM_REG(0x00c4)
#define IPU_DISP_ALT1			IPU_CM_REG(0x00c8)
#define IPU_DISP_ALT2			IPU_CM_REG(0x00cc)
#define IPU_DISP_ALT3			IPU_CM_REG(0x00d0)
#define IPU_DISP_ALT4			IPU_CM_REG(0x00d4)
#define IPU_SNOOP			IPU_CM_REG(0x00d8)
#define IPU_MEM_RST			IPU_CM_REG(0x00dc)
#define IPU_PM				IPU_CM_REG(0x00e0)
#define IPU_GPR				IPU_CM_REG(0x00e4)
#define IPU_CHA_DB_MODE_SEL(ch)		IPU_CM_REG(0x0150 + 4 * ((ch) / 32))
#define IPU_ALT_CHA_DB_MODE_SEL(ch)	IPU_CM_REG(0x0168 + 4 * ((ch) / 32))
#define IPU_CHA_CUR_BUF(ch)		IPU_CM_REG(0x023C + 4 * ((ch) / 32))
#define IPU_ALT_CUR_BUF0		IPU_CM_REG(0x0244)
#define IPU_ALT_CUR_BUF1		IPU_CM_REG(0x0248)
#define IPU_SRM_STAT			IPU_CM_REG(0x024C)
#define IPU_PROC_TASK_STAT		IPU_CM_REG(0x0250)
#define IPU_DISP_TASK_STAT		IPU_CM_REG(0x0254)
#define IPU_CHA_BUF0_RDY(ch)		IPU_CM_REG(0x0268 + 4 * ((ch) / 32))
#define IPU_CHA_BUF1_RDY(ch)		IPU_CM_REG(0x0270 + 4 * ((ch) / 32))
#define IPU_ALT_CHA_BUF0_RDY(ch)	IPU_CM_REG(0x0278 + 4 * ((ch) / 32))
#define IPU_ALT_CHA_BUF1_RDY(ch)	IPU_CM_REG(0x0280 + 4 * ((ch) / 32))

#define IPU_INT_CTRL(n)		IPU_CM_REG(0x003C + 4 * ((n) - 1))
#define IPU_INT_CTRL_IRQ(irq)	IPU_INT_CTRL(((irq) / 32))
#define IPU_INT_STAT_IRQ(irq)	IPU_INT_STAT(((irq) / 32))
#define IPU_INT_STAT(n)		IPU_CM_REG(0x0200 + 4 * ((n) - 1))

extern void __iomem *ipu_cm_reg;

static inline u32 ipu_cm_read(unsigned offset)
{
	return __raw_readl(ipu_cm_reg + offset);
}

static inline void ipu_cm_write(u32 value, unsigned offset)
{
	__raw_writel(value, ipu_cm_reg + offset);
}

#define IPU_IDMAC_REG(offset)	(offset)

#define IDMAC_CONF			IPU_IDMAC_REG(0x0000)
#define IDMAC_CHA_EN(ch)		IPU_IDMAC_REG(0x0004 + 4 * ((ch) / 32))
#define IDMAC_SEP_ALPHA			IPU_IDMAC_REG(0x000c)
#define IDMAC_ALT_SEP_ALPHA		IPU_IDMAC_REG(0x0010)
#define IDMAC_CHA_PRI(ch)		IPU_IDMAC_REG(0x0014 + 4 * ((ch) / 32))
#define IDMAC_WM_EN(ch)			IPU_IDMAC_REG(0x001c + 4 * ((ch) / 32))
#define IDMAC_CH_LOCK_EN_1		IPU_IDMAC_REG(0x0024)
#define IDMAC_CH_LOCK_EN_2		IPU_IDMAC_REG(0x0028)
#define IDMAC_SUB_ADDR_0		IPU_IDMAC_REG(0x002c)
#define IDMAC_SUB_ADDR_1		IPU_IDMAC_REG(0x0030)
#define IDMAC_SUB_ADDR_2		IPU_IDMAC_REG(0x0034)
#define IDMAC_BAND_EN(ch)		IPU_IDMAC_REG(0x0040 + 4 * ((ch) / 32))
#define IDMAC_CHA_BUSY(ch)		IPU_IDMAC_REG(0x0100 + 4 * ((ch) / 32))

extern void __iomem *ipu_idmac_reg;

static inline u32 ipu_idmac_read(unsigned offset)
{
	return __raw_readl(ipu_idmac_reg + offset);
}

static inline void ipu_idmac_write(u32 value, unsigned offset)
{
	__raw_writel(value, ipu_idmac_reg + offset);
}

#define idmac_idma_is_set(reg, dma)	(ipu_idmac_read(reg(dma)) & idma_mask(dma))
#define idma_mask(ch)			(1 << (ch & 0x1f))
#define ipu_idma_is_set(reg, dma)	(ipu_cm_read(reg(dma)) & idma_mask(dma))

enum ipu_modules {
	IPU_CONF_CSI0_EN		= (1 << 0),
	IPU_CONF_CSI1_EN		= (1 << 1),
	IPU_CONF_IC_EN			= (1 << 2),
	IPU_CONF_ROT_EN			= (1 << 3),
	IPU_CONF_ISP_EN			= (1 << 4),
	IPU_CONF_DP_EN			= (1 << 5),
	IPU_CONF_DI0_EN			= (1 << 6),
	IPU_CONF_DI1_EN			= (1 << 7),
	IPU_CONF_SMFC_EN		= (1 << 8),
	IPU_CONF_DC_EN			= (1 << 9),
	IPU_CONF_DMFC_EN		= (1 << 10),

	IPU_CONF_VDI_EN			= (1 << 12),

	IPU_CONF_IDMAC_DIS		= (1 << 22),

	IPU_CONF_IC_DMFC_SEL		= (1 << 25),
	IPU_CONF_IC_DMFC_SYNC		= (1 << 26),
	IPU_CONF_VDI_DMFC_SYNC		= (1 << 27),

	IPU_CONF_CSI0_DATA_SOURCE	= (1 << 28),
	IPU_CONF_CSI1_DATA_SOURCE	= (1 << 29),
	IPU_CONF_IC_INPUT		= (1 << 30),
	IPU_CONF_CSI_SEL		= (1 << 31),
};

struct ipu_channel {
	unsigned int num;

	bool enabled;
	bool busy;
};

ipu_color_space_t format_to_colorspace(u32 fmt);
bool ipu_pixel_format_has_alpha(u32 fmt);

u32 _ipu_channel_status(struct ipu_channel *channel);

int _ipu_chan_is_interlaced(struct ipu_channel *channel);

int ipu_module_enable(u32 mask);
int ipu_module_disable(u32 mask);

int ipu_di_init(struct platform_device *pdev, int id, unsigned long base,
		u32 module, struct clk *ipu_clk);
void ipu_di_exit(struct platform_device *pdev, int id);

int ipu_dmfc_init(struct platform_device *pdev, unsigned long base,
		struct clk *ipu_clk);
void ipu_dmfc_exit(struct platform_device *pdev);

int ipu_dp_init(struct platform_device *pdev, unsigned long base);
void ipu_dp_exit(struct platform_device *pdev);

int ipu_dc_init(struct platform_device *pdev, unsigned long base,
		unsigned long template_base);
void ipu_dc_exit(struct platform_device *pdev);

int ipu_cpmem_init(struct platform_device *pdev, unsigned long base);
void ipu_cpmem_exit(struct platform_device *pdev);

void ipu_get(void);
void ipu_put(void);

#endif				/* __IPU_PRV_H__ */
